/************************************************************************/
/*	Copyright (C) 2005  Michael C. Shultz				*/
/*									*/
/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License, or (at*/
/* your option) any later version.					*/
/*									*/
/* This program is distributed in the hope that it will be useful,	*/
/* but WITHOUT ANY WARRANTY; without even the implied warranty of	*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	*/
/* GNU General Public License for more details. 			*/
/*									*/
/* You should have received a copy of the GNU General Public License	*/
/* along with this program; if not, write to the Free Software		*/
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA		*/
/*  02111-1307, USA.							*/
/* 									*/
/* Michael C. Shultz							*/
/* ringworm01@gmail.com							*/
/* Box 3238 Landers, CA 92285						*/
/************************************************************************/
#include	<libMG.h>
#include	<libMG.ph>

int	MGdbRead( classDb* db )	
{
	FILE*		dbStream;
	char		id[]			= "MGdbRead";
	int		dataBufferIdx		= 0;
	int		dataBufferSize		= 0;
	
	/***************************************************************/
	/* check if we have a valid object                             */
	/***************************************************************/
	if(db == NULL || db->parent.type != CLASS_TYPE_DB)
	{
		fprintf( stderr, "%s error: invalid object type\n", id);
		assert(0);
	}

	/***************************************************************/
	/* set up dataBuffer                                           */
	/***************************************************************/
	dataBufferSize	= MGrFileSize(db->parent.path);
	/*zzzzzzzzzzzzzz*/
	db->parent.mallocIdx++;
	db->dataBuffer	= (char*)malloc(dataBufferSize);
	/*zzzzzzzzzzzzzz*/
	if(!db->dataBuffer)
	{
		fprintf( stderr, "%s error: unable to allocate %d bytes memory for db->dataBuffer\n",
			id, dataBufferSize );
		perror( "system message" );
		assert(0);
	}
	/*zzzzzzzzzzzzzz*/
	db->parent.fopenIdx++;
	dbStream	= fopen( db->parent.path, "r" );
	/*zzzzzzzzzzzzzz*/
	if( !dbStream )
	{
		fprintf( stderr, "%s error: unable to open file %s\n", id, db->parent.path );
		perror( "system message" );
			assert(0);
	}

	fread( db->dataBuffer, dataBufferSize, 1, dbStream );
	if( ferror( dbStream ) )
	{
		fprintf( stderr, "%s error: reading file %s into db->dataBuffer\n", id, db->parent.path );
		perror( "system message" );	\
		assert(0);
	}

	/*
	 *  set db->fieldQty
	 */
	dataBufferIdx	= 0;
	db->fieldQty	= 1;
	while( db->dataBuffer[dataBufferIdx] != 0 || db->dataBuffer[dataBufferIdx+1] != LINEFEED )
	{
		if( db->dataBuffer[dataBufferIdx] == 0 )
		{
			db->fieldQty++;
		}
		dataBufferIdx++;
	}

	/***************************************************************/
	/* set db->parent.recordQty                                    */
	/***************************************************************/
	dataBufferIdx		= 0;
	db->fieldIdx		= 0;
	db->parent.recordQty	= 0;
	while( dataBufferIdx < dataBufferSize )
	{
		if( db->dataBuffer[dataBufferIdx] == 0 )
		{
			db->fieldIdx++;
			if( db->dataBuffer[dataBufferIdx+1] == LINEFEED )
			{
				db->parent.recordQty++;
				if( db->fieldIdx != db->fieldQty )
				{
					fprintf( stderr, "%s error: inconsitant field count at record %d. field count is %d and should be %d\n",
						id, db->parent.recordQty, db->fieldIdx, db->fieldQty );
					assert(0);
				}
				db->fieldIdx	= 0;
			}
		}
		dataBufferIdx++;
	}

	/***************************************************************/
	/* set up record index                                         */
	/***************************************************************/
	/*zzzzzzzzzzzzzz*/
	db->parent.mallocIdx++;
	db->record	= (char***)malloc(db->parent.recordQty*sizeof(char***));
	/*zzzzzzzzzzzzzz*/

	db->fieldIdx		= 0;
	db->parent.recordIdx	= 0;
	while( db->parent.recordIdx < db->parent.recordQty )
	{
		/*zzzzzzzzzzzzzz*/
		db->parent.mallocIdx++;
		db->record[db->parent.recordIdx]	= (char**)malloc(sizeof(char**)*db->fieldQty);	
		/*zzzzzzzzzzzzzz*/
		if( DEBUG_MGdb )
		{
			fprintf( stdout, "%s DEBUG(135): %s db->record[%d] = malloc\n",
				id, db->parent.path, db->parent.recordIdx );
		}
		db->parent.recordIdx++;
	}

	db->record[0][0]	= ( char* )db->dataBuffer;
	db->parent.recordIdx	= 0;
	db->fieldIdx		= 0;
	dataBufferIdx		= 0;

	while( db->parent.recordIdx < db->parent.recordQty )
	{
		/* set record pointers to buffer locations */
		db->record[db->parent.recordIdx][db->fieldIdx]	= (char*)(db->dataBuffer + dataBufferIdx);
		if( DEBUG_MGdb )
		{
			fprintf( stdout, "%s DEBUG(151): %s db->record[%d][%d] = %s\n",
				id, db->parent.path, db->parent.recordIdx, db->fieldIdx,
				db->record[db->parent.recordIdx][db->fieldIdx] );
		}

		/* hunt for end of field */
		while( db->dataBuffer[dataBufferIdx] )	/* while db->dataBuffer[dataBufferIdx] isn't NULL */
		{
			dataBufferIdx++;
			if( !db->dataBuffer[dataBufferIdx] )
			{
				db->fieldIdx++;		/* increment filedIdx if db->dataBuffer[dataBufferIdx] is NULL */
			}
			if( db->dataBuffer[dataBufferIdx + 1] == LINEFEED )
			{
				db->fieldIdx		= 0;	/* set fieldIdx = 0 'cause end of record */
				db->parent.recordIdx++;		/* set to next record 'cause end of previous */
				dataBufferIdx++;		/* now db->dataBuffer[dataBufferIdx] == LINEFEED */
				break;
			}
		}
		dataBufferIdx++;	/* while loop exits with either db->dataBuffer[dataBufferIdx] = NULL or LINEFEED */
	}

	/*zzzzzzzzzzzzzz*/
	db->parent.fopenIdx--;
	fclose( dbStream );
	/*zzzzzzzzzzzzzz*/
	return( 0 );
}


