/************************************************************************/
/*	Copyright (C) 2005  Michael C. Shultz				*/
/*									*/
/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License, or (at*/
/* your option) any later version.					*/
/*									*/
/* This program is distributed in the hope that it will be useful,	*/
/* but WITHOUT ANY WARRANTY; without even the implied warranty of	*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	*/
/* GNU General Public License for more details. 			*/
/*									*/
/* You should have received a copy of the GNU General Public License	*/
/* along with this program; if not, write to the Free Software		*/
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA		*/
/*  02111-1307, USA.							*/
/* 									*/
/* Michael C. Shultz							*/
/* ringworm01@gmail.com							*/
/* Box 3238 Landers, CA 92285						*/
/************************************************************************/
#include	<libMG.h>
#include	<libMG.ph>

/* mode 0: full path/all entries                                             */
/* mode 1: full path/all entries less . and ..                               */
/* mode 2: full path/directory entries less . and ..                         */
/* mode 3: full path/file entries less . and ..                              */
/* mode 4: no path/all entries                                               */
/* mode 5: no path/all entries less . and ..                                 */
/* mode 6: no path/directory entries less . and ..                           */
/* mode 7: no path/file entries less . and ..                                */

classDir*	MGdirCreate( int mode, char* path )
{
	DIR*	dirStream		= NULL;
	FILE*	fileStream;
	char	id[]			= "MGdirCreate";
	char*	fileName;
	int	errorCode		= 0;
	int	idx			= 0;
	int	stringSize		= 0;
	static	classDir*	dir;
	struct	dirent*	dirRecord;

	/*zzzzzzzzzzzzzz*/
	dir			= (classDir*)malloc(sizeof(classDir));
	/*zzzzzzzzzzzzzz*/

	dir->parent.mallocIdx	= 0;
	dir->parent.fopenIdx	= 0;
	dir->parent.mallocIdx++;

	dir->parent.bof		= 1;
	dir->parent.eof		= 0;
	dir->parent.recordIdx	= 0;
	dir->parent.type	= CLASS_TYPE_DIR;

	/*zzzzzzzzzzzzzz*/
	dir->parent.mallocIdx++;
	dir->record		= (char**)malloc(sizeof(char**)*MAXENTRIES);
	/*zzzzzzzzzzzzzz*/

	/*z(1)zzzzzzzzzz*/
	dir->parent.fopenIdx++;
	dirStream	= opendir( path );
	/*zzzzzzzzzzzzzz*/

	if(!dirStream)
	{
		fprintf(stderr, "%s ERROR: could not open dir %s\n", id, path );
		perror("system message");
		assert(0);
	}

	idx			= 0;
	dir->parent.recordQty	= 0;
	dirRecord = readdir(dirStream);
	while( dirRecord )
	{
		switch(mode)
		{
			/* mode 5: no path/all entries less . and .. */
			case	5:
			{
				if(idx > 1)
				{
					/*zzzzzzzzzzzzzz*/
					dir->parent.mallocIdx++;
					/*zzzzzzzzzzzzzz*/
					MGmSetString( dir->record[dir->parent.recordQty], dirRecord->d_name );
					/*zzzzzzzzzzzzzz*/

					dir->parent.recordQty++;
				}
				break;
			}
			/* mode 7: no path/file entries less . and .. */
			case	7:
			{
				if(idx > 1)
				{

					stringSize	= strlen(path)
							+ strlen("/")
							+ strlen(dirRecord->d_name)
							+ 1;

					/*zzzzzzzzzzzzzz*/
					dir->parent.mallocIdx++;
					/*zzzzzzzzzzzzzz*/
					fileName	= (char*)malloc(stringSize);
					/*zzzzzzzzzzzzzz*/

					fileName[0]	= 0;
					strcpy(fileName, path);
					strcat(fileName, "/");
					strcat(fileName, dirRecord->d_name);

					/*z(2)zzzzzzzzzz*/
					dir->parent.fopenIdx++;
					fileStream	= fopen( fileName, "a" );
					/*zzzzzzzzzzzzzz*/

					/* if fileStream can be opened mode "a" then it is a file */
					if( fileStream )
					{
						/*zzzzzzzzzzzzzz*/
						dir->parent.mallocIdx--;
						free(fileName);
						/*zzzzzzzzzzzzzz*/

						/*z(2)zzzzzzzzzz*/
						dir->parent.fopenIdx--;
						fclose(fileStream);
						/*zzzzzzzzzzzzzz*/

						break;
					}
					/*z(2)zzzzzzzzzz*/
					dir->parent.fopenIdx--;
					/*zzzzzzzzzzzzzz*/
					/*zzzzzzzzzzzzzz*/
					dir->parent.mallocIdx--;
					free(fileName);
					/*zzzzzzzzzzzzzz*/

					/*zzzzzzzzzzzzzz*/
					dir->parent.mallocIdx++;
					MGmSetString( dir->record[dir->parent.recordQty], dirRecord->d_name );
					/*zzzzzzzzzzzzzz*/

					if( errorCode )
					{
						fprintf( stderr, "%s error: %s returned errorCode %d\n",
							id, "MGmSetString", errorCode );
						assert(0);
					}
					dir->parent.recordQty++;
				}
				break;
			}
			default:
			{
				fprintf( stderr, "%s error: unknown mode %d\n", id, mode);

				dir->parent.mallocIdx--;
				free(dir->record);

				dir->parent.fopenIdx--;
				closedir(dirStream);
				assert(0);
			}
		}
		idx++;
		dirRecord = readdir(dirStream);
	}

	/*z(1)zzzzzzzzzz*/
	dir->parent.fopenIdx--;
	closedir(dirStream);
	/*zzzzzzzzzzzzzz*/

	return(dir);
}
