/************************************************************************/
/*	Copyright (C) 2005  Michael C. Shultz				*/
/*									*/
/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License, or (at*/
/* your option) any later version.					*/
/*									*/
/* This program is distributed in the hope that it will be useful,	*/
/* but WITHOUT ANY WARRANTY; without even the implied warranty of	*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	*/
/* GNU General Public License for more details. 			*/
/*									*/
/* You should have received a copy of the GNU General Public License	*/
/* along with this program; if not, write to the Free Software		*/
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA		*/
/*  02111-1307, USA.							*/
/* 									*/
/* Michael C. Shultz							*/
/* ringworm01@gmail.com							*/
/* Box 3238 Landers, CA 92285						*/
/************************************************************************/
#include	<libPMGR.h>

int	PMGRrDbCreate( structProperty* property )
{
	char		id[]			= "PMGRrDbCreate"; 
	char		pkgDir[]		= PKGDBDIR;
	char*		fieldKey		= NULL;
	char*		fieldValue		= NULL;
	char*		installedPortDir	= NULL;
	char*		installedPortName	= NULL;
	classDir*	dir			= NULL;
	int		IDX_configDb		= 1;
	int		QTY_configDb		= 0;
	int		dirMode			= 7;
	int		errorCode		= 0;
	signed int	fopenIdx		= 0;
	signed int	mallocIdx		= 0;

	property->objIdx	= 0;

	/***************************************************************/
	/* add any ignored ports in config.db  to ignore.db            */
	/***************************************************************/
	IDX_configDb	= 1;
	QTY_configDb	= MGdbGetRecordQty( property->configDb );
	
	
	while( IDX_configDb < QTY_configDb )
	{
		fieldKey	= MGdbGet( property->configDb, IDX_configDb, property->fieldKey );
		if( !strcmp( fieldKey, "IGNORE" ) )
		{
			fieldValue	= MGdbGet( property->configDb, IDX_configDb, property->fieldValue );
			MGdbGoTop( property->ignoreDb );
			if( !MGdbSeek( property->ignoreDb, property->fieldIgnorePortDir, fieldValue,
					property->fieldIgnorePortDir ) )
			{
				fprintf( stdout, "%s %s info: adding %s to ignore.db reason: pm-020.conf option\n",
					id, VERSION, fieldValue );
				MGdbAdd( property->ignoreDb, fieldValue, "pm-020.conf option", NULL );
			}
		}
		IDX_configDb++;	
	}

	/***************************************************************/
	/* create initial data bases                                   */
	/***************************************************************/

	/*zzzzzzzzzzzzzz*/
	property->objIdx++;
	property->installedPortsDb	= MGdbCreate(
						property->installedPortsDbFileName,
						property->fieldInstalledPortName,	/* contains the name of field #0 */
						property->fieldInstalledPortDir,	/* contains the name of field #1 */
						NULL);
	/*zzzzzzzzzzzzzz*/

	/*zzzzzzzzzzzzzz*/
	property->objIdx++;
	property->dependencyPortsDb	= MGdbCreate( 
						property->dependencyPortsDbFileName,
						property->fieldChildPortName,
						property->fieldDependencyPortName,
						property->fieldDependencyPortDir,
						NULL );
	/*zzzzzzzzzzzzzz*/

	/*zzzzzzzzzzzzzz*/
	property->objIdx++;
	property->outOfDatePortsDb	= MGdbCreate(
						property->oldPortsFileName,
						property->fieldOldPortDir,
						property->fieldOldPortName,
						property->fieldReason,
						property->fieldNewPortName,
						NULL );
	/*zzzzzzzzzzzzzz*/
	
	if( !MGrIfFileExist( property->cacheDbFileName ) )
	{
		/*zzzzzzzzzzzzzz*/
		property->objIdx++;
		property->cacheDb	= MGdbCreate( property->cacheDbFileName,
							property->fieldCachePortDir,
							property->fieldAvailablePortName,
							property->fieldMakeFileSize,
							property->fieldMakeFileTime,
							NULL );
		/*zzzzzzzzzzzzzz*/
	}
	else
	{
		/*zzzzzzzzzzzzzz*/
		property->objIdx++;
		property->cacheDb	= MGdbOpen( property->cacheDbFileName );	
		/*zzzzzzzzzzzzzz*/
	}
						

	/*zzzzzzzzzzzzzz*/
	property->objIdx++;
	dir				= MGdirCreate( dirMode, pkgDir );
	/*zzzzzzzzzzzzzz*/

	/***************************************************************/
	/* fill property->installedPortsDb records with                */
	/* installedPortName, installedPortDir records                 */
	/***************************************************************/
	installedPortName		= MGdirGet(dir);

	errorCode			= PMGRrVerifyContentsFile( installedPortName );
	if(errorCode)
	{
		fprintf(stderr, "%s %s error: %s returned errorCode %d\n",
			id, VERSION, "PMGRrVerifyContentsFile", errorCode);
		assert(0);
	}

	/***************************************************************/
	/* fill property->dependencyPortsDb records with               */
	/* installedPortName, dependencyPortName, dependencyPortDir    */
	/***************************************************************/
	while( installedPortName && !errorCode)
	{
		PMGRrAddDependencies( property, installedPortName  );
		installedPortDir	= PMGRrGetPortDir( installedPortName );
		MGdbAdd( property->installedPortsDb, installedPortName, installedPortDir, NULL);

		MGdirSkip(dir);

		installedPortName	= MGdirGet(dir);

		if(!installedPortName)
		{
			break;
		}

		errorCode	= PMGRrVerifyContentsFile(installedPortName);

		if(errorCode)
		{
			fprintf(stderr, "%s %s error: %s returned errorCode %d\n",
				id, VERSION, "PMGRrVerifyContentsFile", errorCode);
			assert(0);
		}
	}

	/*zzzzzzzzzzzzzz*/
	property->objIdx--;
	MGdirDestroy( dir );
	/*zzzzzzzzzzzzzz*/

	/*
	 *	create port objects
	 */
	

	if(fopenIdx || mallocIdx)
	{
		fprintf( stderr, "%s %s error: fopenIdx = %d mallocIdx = %d\n",	id, VERSION, fopenIdx, mallocIdx );
		assert(0);
	}
	return(errorCode);
}
