/************************************************************************/
/*	Copyright (C) 2005  Michael C. Shultz				*/
/*									*/
/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License, or (at*/
/* your option) any later version.					*/
/*									*/
/* This program is distributed in the hope that it will be useful,	*/
/* but WITHOUT ANY WARRANTY; without even the implied warranty of	*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	*/
/* GNU General Public License for more details. 			*/
/*									*/
/* You should have received a copy of the GNU General Public License	*/
/* along with this program; if not, write to the Free Software		*/
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA		*/
/*  02111-1307, USA.							*/
/* 									*/
/* Michael C. Shultz							*/
/* ringworm01@gmail.com							*/
/* Box 3238 Landers, CA 92285						*/
/************************************************************************/
#include	"libPMGR.h"


#define PORTDIR_F0	0
#define PORTNAME_F1	1
#define MAKEFILESIZE_F2 2
#define MAKEFILETIME_F4 4

char*	rGetFromMakeDescribe( structProperty* property, char* installedPortDir );

char*	PMGRrMakeDescribe( structProperty* property, char* installedPortDir )
{
	char	id[]				= "PMGRrMakeDescribe";
	char*	available			= NULL;
	char*	strMakeFile			= NULL;
	char*	strMakeFileSize			= NULL;
	char*	strMakeFileSizeTemp		= NULL;
	char*	strMakeFileTime			= NULL;
	char*	strMakeFileTimeTemp		= NULL;
	int	cacheIdx			= 0;
	int	cacheQty			= 0;
	int	errorCode			= 0;
	int	fopenIdx			= 0;
	int	makeFileSize			= 0;
	int	mallocIdx			= 0;

	makeFileSize	= strlen( PORTSDIR ) + strlen( installedPortDir ) + strlen( "/Makefile" ) + 1;

	/*zzzzzzzzzzzzzz*/
	mallocIdx++;
	strMakeFile	= ( char* )malloc( makeFileSize );
	/*zzzzzzzzzzzzzz*/

	strMakeFile[0]	= 0;
	strcpy( strMakeFile, PORTSDIR );
	strcat( strMakeFile, installedPortDir );
	strcat( strMakeFile, "/Makefile" );

	cacheQty	= MGdbGetRecordQty( property->cacheDb );

	/* if Makefile is missing, return */
	if( !MGrIfFileExist(strMakeFile) )
	{
		free(strMakeFile);
		/* need to delete the cache record here */
		cacheIdx	= 0;
		while( cacheIdx < cacheQty )
		{
			if( !strcmp( installedPortDir, MGdbGet( property->cacheDb, cacheIdx, 
								property->fieldCachePortDir ) ) )
			{
				property->cacheDb	= MGdbDelete( property->cacheDb, cacheIdx );
				return( NULL );
			}
			cacheIdx++;
		}
		return( NULL );
	}
	strMakeFileSizeTemp	= MGrIntToString( MGrFileSize( strMakeFile ) );
	/*zzzzzzzzzzzzzz*/
	mallocIdx++;
	MGmSetString( strMakeFileSize, strMakeFileSizeTemp );
	/*zzzzzzzzzzzzzz*/

	strMakeFileTimeTemp	= MGrIntToString( (int)MGrFileTime( strMakeFile ) );

	/*zzzzzzzzzzzzzz*/
	mallocIdx++;
	MGmSetString( strMakeFileTime, strMakeFileTimeTemp );
	/*zzzzzzzzzzzzzz*/

	cacheIdx	= 0;

	while( cacheIdx < cacheQty )
	{
		/* if installedPortDir (passed on calling PMGRrMakeDescribe) == a portDir in the cache db */
		if( !strcmp( installedPortDir, MGdbGet( property->cacheDb, cacheIdx, property->fieldCachePortDir ) ) )
		{
			/* and if the Makefile size for installedPortDir is the same size as in the cache db */
			if( !strcmp( strMakeFileSize, MGdbGet( property->cacheDb, cacheIdx, property->fieldMakeFileSize ) ) ) 
			{
				/* and if the Makefile time of creation is the same as in the cache db */
				if( !strcmp( strMakeFileTime, MGdbGet( property->cacheDb, cacheIdx, property->fieldMakeFileTime ) ) ) 
				{
					/*zzzzzzzzzzzzzz*/
					mallocIdx--;
					free( strMakeFileSize );
					/*zzzzzzzzzzzzzz*/

					/*zzzzzzzzzzzzzz*/
					mallocIdx--;
					free( strMakeFileTime );
					/*zzzzzzzzzzzzzz*/

					/*zzzzzzzzzzzzzz*/
					mallocIdx--;
					free( strMakeFile );
					/*zzzzzzzzzzzzzz*/

					if( fopenIdx || mallocIdx)
					{
						fprintf( stderr, "%s error: fopenIdx = %d mallocIdx = %d\n",
							id, fopenIdx, mallocIdx );
						assert(0);
					}
					return( MGdbGet( property->cacheDb, cacheIdx, property->fieldAvailablePortName ) );
				}
				else
				{
					/* if here then Makefile time is wrong  so delete the cache db record */
					property->cacheDb	= MGdbDelete( property->cacheDb, cacheIdx );
					cacheIdx		= cacheQty;
					break;
				}
			}
			else
			{
				/* if here then Makefile size is wrong so delete the record */
				property->cacheDb	= MGdbDelete( property->cacheDb, cacheIdx );
				cacheIdx		= cacheQty;
				break;
			}
		}
		cacheIdx++;
	}

	/************************/
	/* cache miss		*/
	/************************/
	fprintf( stdout, "(Cache miss)" );
	available	= rGetFromMakeDescribe( property, installedPortDir );
	MGdbAdd( property->cacheDb, installedPortDir, available, strMakeFileSize, strMakeFileTime, NULL );
	free(available);

	/*zzzzzzzzzzzzzz*/
	mallocIdx--;
	free( strMakeFileSize );
	/*zzzzzzzzzzzzzz*/

	/*zzzzzzzzzzzzzz*/
	mallocIdx--;
	free( strMakeFileTime );
	/*zzzzzzzzzzzzzz*/

	/*zzzzzzzzzzzzzz*/
	mallocIdx--;
	free( strMakeFile );
	/*zzzzzzzzzzzzzz*/

	if( fopenIdx || mallocIdx)
	{
		fprintf( stderr, "%s error: fopenIdx = %d mallocIdx = %d\n",
			id, fopenIdx, mallocIdx );
		assert(0);
	}
	/* the cache record we just added will be the last one in the db */
	return( MGdbGet( property->cacheDb, MGdbGetRecordQty( property->cacheDb )-1, property->fieldAvailablePortName ) );
}

/* get correct portname by running "make {options if any from pm-020.conf} describe" in portname's directory */
char*	rGetFromMakeDescribe( structProperty* property, char* installedPortDir )
{
	FILE*		tempStream		= NULL;
	char		id[]			= "rGetFromMakeDescribe";
	char*		currentPortMakefile	= NULL;
	int		cacheRecordSize		= 65000;
	int		fopenIdx		= 0;
	int		idx			= 0;
	int		mallocIdx		= 0;
	int		miscPtr 		= 0;
	int		skip			= 0;
	int		stringSize		= 0;
	static	char*	cacheRecord		= NULL;

	/*zzzzzzzzzzzzzz*/
	mallocIdx++;
	cacheRecord	= calloc( cacheRecordSize, 1 );
	/*zzzzzzzzzzzzzz*/

	while( idx < MGdbGetRecordQty( property->configDb ) )
	{
		/* see if this port has anything special in configDb */
		if( !strcmp( installedPortDir, MGdbGet( property->configDb, idx, property->fieldKey ) ) )
		{
			stringSize	= strlen( "( cd " )
					+ strlen( PORTSDIR )
					+ strlen( installedPortDir )
					+ strlen( "; make " )
					+ strlen( MGdbGet( property->configDb, idx, property->fieldValue ) )
					+ strlen( " describe )" )
					+ 1;

			/*zzzzzzzzzzzzzz*/
			mallocIdx++;
			currentPortMakefile	= ( char* )malloc( stringSize );
			/*zzzzzzzzzzzzzz*/
			currentPortMakefile[0]	= 0;
			strcpy( currentPortMakefile, "( cd " );
			strcat( currentPortMakefile, PORTSDIR );
			strcat( currentPortMakefile, installedPortDir );
			strcat( currentPortMakefile, "; make " );
			strcat( currentPortMakefile, MGdbGet( property->configDb, idx, property->fieldValue ) );
			strcat( currentPortMakefile, " describe )" );
			skip	= 1;
			break;
		}
		idx++;
	}
	if( skip == 0 )
	{
		/*	set get portname for Makefile lookup from portI.db	*/

			stringSize	= strlen( "( cd " )
					+ strlen( PORTSDIR )
					+ strlen( installedPortDir )
					+ strlen( "; make " )
					+ strlen( " describe )" )
					+ 1;

			/*zzzzzzzzzzzzzz*/
			mallocIdx++;
			currentPortMakefile	= ( char* )malloc( stringSize );
			/*zzzzzzzzzzzzzz*/
			currentPortMakefile[0]	= 0;
			strcpy( currentPortMakefile, "( cd " );
			strcat( currentPortMakefile, PORTSDIR );
			strcat( currentPortMakefile, installedPortDir );
			strcat( currentPortMakefile, "; make " );
			strcat( currentPortMakefile, " describe )" );

	}
	/*zzzzzzzzzzzzzz*/
	fopenIdx++;
	tempStream	= popen( currentPortMakefile, "r" );
	/*zzzzzzzzzzzzzz*/

	if( ferror( tempStream ) )
	{
		fprintf( stderr, "%s error: reading make describe into cacheRecord\n", id );
		perror( "system message" );
		assert(0);
	}
	fread( cacheRecord, cacheRecordSize, 1, tempStream );
	if(DEBUG_PMGRrMakeDescribe)
	{
		fprintf( stderr, "DEBUG (252): %s: currentPortMakefile = %s\n", id, currentPortMakefile );
	}

	/*zzzzzzzzzzzzzz*/
	fopenIdx--;
	pclose( tempStream );
	/*zzzzzzzzzzzzzz*/

	/*zzzzzzzzzzzzzz*/
	mallocIdx--;
	free( currentPortMakefile );
	/*zzzzzzzzzzzzzz*/

	miscPtr = 0;
	while( miscPtr < (int)strlen( cacheRecord ) )
	{
		if( cacheRecord[miscPtr] == '|' )
		{
			cacheRecord[miscPtr] = 0;
			cacheRecord = realloc( cacheRecord, miscPtr + 1 );
			miscPtr = ( int )strlen( cacheRecord );
		}
		miscPtr++;
	}
	cacheRecord	= ( char* )realloc( cacheRecord, strlen( cacheRecord ) + 1 );

	if( fopenIdx || mallocIdx != 1)
	{
		fprintf( stderr, "%s error: fopenIdx = %d mallocIdx = %d\n",
			id, fopenIdx, mallocIdx );
		assert(0);
	}
	return( cacheRecord );
}
