/************************************************************************/
/*	Copyright (C) 2005  Michael C. Shultz				*/
/*									*/
/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License, or (at*/
/* your option) any later version.					*/
/*									*/
/* This program is distributed in the hope that it will be useful,	*/
/* but WITHOUT ANY WARRANTY; without even the implied warranty of	*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	*/
/* GNU General Public License for more details. 			*/
/*									*/
/* You should have received a copy of the GNU General Public License	*/
/* along with this program; if not, write to the Free Software		*/
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA		*/
/*  02111-1307, USA.							*/
/* 									*/
/* Michael C. Shultz							*/
/* ringworm01@gmail.com							*/
/* Box 3238 Landers, CA 92285						*/
/************************************************************************/

#include	<libPMGR.h>
int	PMGRrStatus(structProperty* property)
{
	char	id[]				= "PMGRrStatus";
	char*	available			= NULL;
	char*	command				= NULL;
	char*	dependencyPortDir		= NULL;
	char*	dependencyPortName		= NULL;
	char*	installedPortDir		= NULL;
	char*	installedPortName		= NULL;
	char*	oldPortDir			= NULL;
	char*	oldPortName			= NULL;
	int	IDX_dependencyPortsDb		= 0;
	int	IDX_installedPortsDb		= 1;
	int	QTY_dependencyPortsDb		= 0;
	int	QTY_installedPortsDb		= 0;
	int	answer				= 0;		
	int	errorCode			= 0;
	int	fopenIdx			= 0;	
	int	idx				= 0;
	int	mallocIdx			= 0;
	int	stringSize			= 0;
	
	/*.............................................................*/
	fprintf( stdout,"%s\n", SINGLE_LINES );
	fprintf( stdout,"%s %s info: Creating inital data bases\n", id, VERSION );
	fprintf( stdout,"%s\n", SINGLE_LINES );
	while( fflush( stdout ) );

	errorCode		= PMGRrDbCreate(property);	/* create/open initial databases */
	if(errorCode)
	{
		exit(0);
	}
	/*.............................................................*/
	fprintf( stdout,"%s\n", SINGLE_LINES );
	fprintf( stdout,"%s %s info: looking for missing dependent ports\n", id, VERSION );
	fprintf( stdout,"%s\n", SINGLE_LINES );

	while( fflush( stdout ) );

	errorCode		= PMGRrMissingDependencies( property );
	if(errorCode)
	{
		exit(0);
	}
	/*.............................................................*/
	fprintf( stdout,"%s\n", SINGLE_LINES );
	fprintf( stdout,"%s %s info: looking for old installed ports\n", id, VERSION );
	fprintf( stdout,"%s\n", SINGLE_LINES );

	while( fflush( stdout ) );

	/* compare installedPortsDb records to "make describe" of each installed port  */
	/* remove ports that no longer have a Makefile (deleted by CVSand in /usr/ports/MOVED hopefully ) */
	/* add ports whose installedPortsDb names do not match their "make describe" name to outOfDatePortsDb */

	IDX_installedPortsDb	= 1;	/* start at one to skip over field names */
	QTY_installedPortsDb	= MGdbGetRecordQty( property->installedPortsDb );
	while( IDX_installedPortsDb < QTY_installedPortsDb )
	{
		installedPortDir	= MGdbGet( property->installedPortsDb, IDX_installedPortsDb,
						property->fieldInstalledPortDir);

		/* PMGRrMakeDescribe returns the version of the available port from "cd /usr/ports/; make describe" */
		/*
			change ver 0.2.3 in order to catch changes where the port mantainer forgot to bump revision
			numbers  PMGRrMakeDescribe now sets a change flag which means that even though the port #
			is the same, the make file is either a different size or created at a different date than
			it was when the port was installed
		*/
		available = PMGRrMakeDescribe( property, installedPortDir ); 

		/* remove ports that have been deleted from or moved in the ports collection */		
		/* added 0.1.7 20040804 */
		if( !available )
		{
			installedPortName	= 
				MGdbGet( property->installedPortsDb, IDX_installedPortsDb, property->fieldInstalledPortName);
			fprintf( stdout,"%s\n", SINGLE_LINES );

			fprintf( stdout, 
				"%s is no longer in the ports collection, see /usr/ports/MOVED\nis it OK to remove %s from your system? [y/n]\n",
				installedPortDir, installedPortName );
			answer	= getc(stdin);

			while( fflush( stdout ) );

			if( answer != 89 && answer != 121 )
			{
				fprintf( stdout, "answer	= %d\n", answer );
				fprintf( stdout, "portmanager shutting down\n" );
				exit(0);
			}
			ungetc(answer, stdin);

			fprintf( stdout, "removing: %s\n", installedPortName );
			stringSize	= strlen("pkg_delete -f ") + strlen(installedPortName) + 1;
			command 	= ( char* )malloc( stringSize );
			command[0]	= 0;
			strcpy( command, "pkg_delete -f " );
			strcat( command, installedPortName );
			fprintf( stdout,"%s\n", command );
			fprintf( stdout,"%s\n", SINGLE_LINES );
			system( command );
			free( command );
			/* since installedPortName is no longer in ports collection need to delete record and restart */
			property->installedPortsDb	= MGdbDelete( property->installedPortsDb, IDX_installedPortsDb );
			QTY_installedPortsDb	= MGdbGetRecordQty( property->installedPortsDb );
			IDX_installedPortsDb	= 1;
			while( fflush( stdout ) );
			continue;
		}

		/* compare installed port name with name from "make describe" */
		/* if installed != available then add portname to outOfDatePortsDb */
		installedPortName	= 
			MGdbGet( property->installedPortsDb, IDX_installedPortsDb, property->fieldInstalledPortName);
		installedPortDir	= 
			MGdbGet( property->installedPortsDb, IDX_installedPortsDb, property->fieldInstalledPortDir);
		if( strcmp( installedPortName,	available ) )
		{
			fprintf( stdout,"have:%-25s status: OLD available:%-25s %-25s\n",
				installedPortName, available, installedPortDir );
			while( fflush( stdout ) );

			MGdbAdd( property->outOfDatePortsDb, installedPortDir, installedPortName, "OLD", available, NULL );
			IDX_installedPortsDb++;
			continue;
		}		
		fprintf( stdout,"have:%-25s status: CURRENT: %-25s\n", installedPortName, installedPortDir );
		while( fflush( stdout ) );
		IDX_installedPortsDb++;
	}

	/*.............................................................*/
	fprintf( stdout,"%s\n", SINGLE_LINES );
	fprintf( stdout,"%s %s info: looking for installed ports built with old dependencies\n", id, VERSION );
	fprintf( stdout,"%s\n", SINGLE_LINES );
	while( fflush( stdout ) );
	/****************************************************************************/
	/* add any DependencyDir == installedDir && dependencyName != installedName */
	/* to outOfDatePortsDb                                                      */
	/****************************************************************************/
	IDX_dependencyPortsDb	= 1;
	fprintf( stdout,"%s\n", SINGLE_LINES );
	while( fflush( stdout ) );
	QTY_dependencyPortsDb	= MGdbGetRecordQty( property->dependencyPortsDb);
	while( IDX_dependencyPortsDb < QTY_dependencyPortsDb )
	{
		dependencyPortDir	= MGdbGet( property->dependencyPortsDb, IDX_dependencyPortsDb,
						property->fieldDependencyPortDir);
						
		dependencyPortName	= MGdbGet( property->dependencyPortsDb, IDX_dependencyPortsDb,
						property->fieldDependencyPortName);

		MGdbGoTop( property->installedPortsDb );
		installedPortName	= MGdbSeek( property->installedPortsDb, property->fieldInstalledPortDir,
						dependencyPortDir, property->fieldInstalledPortName );

		idx			= MGdbGetRecno(	property->installedPortsDb ) - 1;

		installedPortDir	= MGdbGet( property->installedPortsDb, idx, property->fieldInstalledPortDir );

		if(installedPortName)	/* KEEP! if this is NULL then next line (strmp installedPortName) will segfault */
		{
			/* if installedPortName != dependencyPortName && DependencyDir == installedPortDir */
			if( strcmp( installedPortName, dependencyPortName ) && !strcmp( dependencyPortDir, installedPortDir ) ) 
			{
				MGdbGoTop( property->dependencyPortsDb );
				while( ( oldPortName = MGdbSeek( property->dependencyPortsDb,
								property->fieldDependencyPortName,
								dependencyPortName, property->fieldChildPortName )
					 ) )
				{
					/*
					 * reSync everything based on oldPortName
					 */
					idx			= MGdbGetRecno( property->dependencyPortsDb ) - 1;
					dependencyPortName	= MGdbGet( property->dependencyPortsDb, idx,
									property->fieldDependencyPortName );
					dependencyPortDir	= MGdbGet( property->dependencyPortsDb, idx,
									property->fieldDependencyPortDir );

					MGdbGoTop( property->installedPortsDb );
					installedPortName	= MGdbSeek( property->installedPortsDb,
									property->fieldInstalledPortDir,
									dependencyPortDir,
									property->fieldInstalledPortName );
					/*
					 * After syncing, dependencyPortName may equal installedPortName, in
					 * this case we need to break out of this while loop
					 */
					 if( !strcmp( dependencyPortName, installedPortName ) )
					 {
					 	break;
					 }
					/*
					 * Add record to outOfDatePortsDb Only if it is unique
					 */
					MGdbGoTop( property->outOfDatePortsDb );
					if( !MGdbSeek( property->outOfDatePortsDb, property->fieldOldPortName,
						oldPortName, property->fieldOldPortName ) )
					{
						fprintf( stdout,"OLD %s built with old dependency %s, current dependency is %s\n",
							oldPortName, dependencyPortName, installedPortName );
						MGdbGoTop( property->installedPortsDb );
						oldPortDir	= MGdbSeek( property->installedPortsDb, 
									property->fieldInstalledPortName,
									oldPortName,
									property->fieldInstalledPortDir );

						MGdbAdd( property->outOfDatePortsDb, oldPortDir, oldPortName,
							"OLD dependency", dependencyPortName, NULL );
						while( fflush( stdout ) );
					}
				}
			}
		}
		++IDX_dependencyPortsDb;
	}

	fprintf( stdout,"%s\n", SINGLE_LINES );
	fprintf( stdout,"status report finished\n" ); 
	fprintf( stdout,"%s\n", DOUBLE_LINES );
	while( fflush( stdout ) );

	if( fopenIdx || mallocIdx != 0)
	{
		fprintf( stdout, "%s error: fopenIdx = %d mallocIdx = %d\n",
			id, fopenIdx, mallocIdx );
		while( fflush( stdout ) );
		assert(0);
	}
	PMGRrDbDestroy( property );
 	return( 0 );
}
