/************************************************************************/
/*      Copyright (C) 2005  Michael C. Shultz                           */
/*                                                                      */
/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License, or (at*/
/* your option) any later version.                                      */
/*                                                                      */
/* This program is distributed in the hope that it will be useful,      */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */
/*                                                                      */
/* You should have received a copy of the GNU General Public License    */
/* along with this program; if not, write to the Free Software          */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA            */
/*  02111-1307, USA.                                                    */
/*                                                                      */
/* Michael C. Shultz                                                    */
/* ringworm01@gmail.com                                                 */
/* Box 3238 Landers, CA 92285                                           */
/************************************************************************/

/*
 *	$Author: mike $
 *	$Revision: 1.3 $
 *	$Date: 2005/02/21 17:35:40 $
 *	$Id: libPMGR.h,v 1.3 2005/02/21 17:35:40 mike Exp $
 *	$Log: libPMGR.h,v $
 *	Revision 1.3  2005/02/21 17:35:40  mike
 *	PMGRrDoUpgrade:  Added make check-conflicts so we no longer waste
 *	time trying to build a port that is going to fail at installation
 *	time. If there is a conflict port is pkg_delete -f'ed.
 *	
 */

#ifndef __LIBPMGR_H__
#define __LIBPMGR_H__	1

#include	<externVars.h>
#include	<libMG.h>

#define	DEBUG_PMGRrDbCreate		0
#define	DEBUG_PMGRrMakeDescribe		0
#define	DEBUG_PMGRrReadConfigure	0
#define	DEBUG_PMGRrStatus		0
#define	DEBUG_PMGRrMissingDependencies	0

/****************************************/
/* SHAREDIR defined in bsd.port.mk	*/ 
/* PKGDBDIR defined in bsd.port.mk	*/
/* data_DATA see automake manual	*/
/****************************************/

/*
 * The following are defined in <externVars.h>
 *
 * BINDIR
 * ETCDIR
 * PKGDBDIR
 * PORTSDIR
 * SHAREDIR
 * TEMPDIR
 *
 */

#define	COMMANDLINE_DB		"/commandLine.db"
#define	CONFIGURE_DB		"/configure.db"
#define	HELPFILE		"/help.txt"
#define	IGNORE_DB		"/ignore.db"
#define	PORTS_CACHE_DB		"/ports_cache.db"
#define	PORTS_INSTALLED_DB	"/ports_installed.db"
#define	PORTS_OLD_DB		"/ports_old.db"
#define	PORT_DEPENDENCIES_DB	"/port_dependencies.db"
#define	STRIKES_DB		"/3strikes.db"
#define CONFIGURE_CONF		"/pm-020.conf"

typedef	struct classPort classPort;

typedef struct
{
	char*		cacheDbFileName;
	char*		commandLineDbFileName;
	char*		configConfFileName;
	char*		configDbFileName;
	char*		dependencyPortsDbFileName;
	char*		helpFile;
	char*		ignoreDbFileName;
	char*		installedPortsDbFileName;
	char*		oldPortsFileName;
	char*		strikesDbFileName;

	char*		field3strikesPortDir;		/* 3strikesDb field 0 */
	char*		fieldStrikes;			/* 3strikesDb field 1 */

	char*		fieldCommandLineKey;		/* commandLineDb field 0 */
	char*		fieldCommandLineValue;		/* commandLineDb field 1 */

	char*		fieldIgnorePortDir;		/* ignoreDb field 0 */
	char*		fieldIgnoreReason;		/* ignoreDb field 1 */

	char*		fieldOldPortDir;		/* outOfDatePortsDb field 1 */
	char*		fieldOldPortName;		/* outOfDatePortsDb field 0 */
	char*		fieldReason;			/* outOfDatePortsDb field 2 */
	char*		fieldNewPortName;		/* outOfDatePortsDb field 3 */

	char*		fieldKey;			/* configDb field 0 */
	char*		fieldValue;			/* configDb field 1 */

	char*		fieldChildPortName;		/* dependencyPortsDb field 0 */
	char*		fieldDependencyPortName;	/* dependencyPortsDb field 1 */
	char*		fieldDependencyPortDir;		/* dependencyPortsDb field 2 */

	char*		fieldCachePortDir;		/* cacheDb field 0 */
	char*		fieldAvailablePortName;		/* cacheDb field 1 */
	char*		fieldMakeFileSize;		/* cacheDb field 2 */
	char*		fieldMakeFileTime;		/* cacheDb field 3 */

	char*		fieldInstalledPortName;		/* installedPortsDb field 0 */
	char*		fieldInstalledPortDir;		/* installedPortsDb field 1 */

	char*		fieldUpgradePortDir;
	char*		fieldUpgradePortName;

	classDb*	strikesDb;
	classDb*	cacheDb;
	classDb*	commandLineDb;
	classDb*	configDb;
	classDb*	dependencyPortsDb;
	classDb*	ignoreDb;
	classDb*	installedPortsDb;
	classDb*	outOfDatePortsDb;

	classPort*	portTree;

	int		objIdx;
	int		portManagerUpdated;		/* 0=not updated 1=updated*/
	int		verbose;
} structProperty;

typedef	struct
{
	char*	msg;
	int	errorIndicator;
	int	errorTest;
	int	messageIndicator;
} structFeedBack;	


char*		PMGRrGetPortDir(char* portName);
char*		PMGRrMakeDescribe( structProperty* property, char* installedPortDir );
classPort*	PMGRportCreate( structFeedBack* feedBack, int argc, char** argv );
int		PMGRportDestroy( classPort* portTree );
int		PMGRportTest( classPort* portTree );
int		PMGRrAddDependencies( structProperty* property, char* portName );
int		PMGRrDbCreate( structProperty* property );
int		PMGRrDbDestroy( structProperty* property );
int		PMGRrDoUpgrade( structProperty* property, char* oldPortName );
int		PMGRrInitialize( structProperty* property, int argc, char** argv );
int		PMGRrMissingDependencies(structProperty* property);
int		PMGRrReadConfigure( structProperty* property );
int		PMGRrShowLeaves( structProperty* property );
int		PMGRrShowLeavesDelete( structProperty* property );
int		PMGRrStatus(structProperty* property);
int		PMGRrUpgrade(structProperty* property);
int		PMGRrVerifyContentsFile(char* portName);

#endif
